@@ -10,7 +10,7 @@ Dancer::Plugin::Database - easy database connections for Dancer applications
 
 =cut
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 my $dbh;
 my $last_connection_check;
@@ -27,6 +27,10 @@ register database => sub {
                 $last_connection_check = time;
                 return $dbh;
             } else {
+                Dancer::Logger->debug(
+                    "Database connection went away, reconnecting"
+                );
+                if ($dbh) { $dbh->disconnect; }
                 return $dbh = _get_connection();
             }
         }
@@ -77,11 +81,15 @@ sub _check_connection {
             return 1;
         } else {
             # It was "0 but true", meaning the default DBI ping implementation
-            # TODO: determine what to do here; try 'select 1' or something?
-            # For now, play it safe and claim the connection is dead, so we
-            # reconnect
-            return;
+            # Implement our own basic check, by performing a real simple query.
+            my $ok;
+            eval {
+                $ok = $dbh->do('select 1');
+            };
+            return $ok;
         }
+    } else {
+        return;
     }
 }
 
